# -*- coding: utf-8 -*-
"""
@author: Manuel
Loads json file with trial data
Creates new variables and remove trials with high data loss.
sound_params.csv --> contains sound length and validation parameters.
Concatenates and returns spd_full_dataset.csv
This output is the input of spd_analysis
"""

import pandas as pd
import json
import matplotlib.pyplot as plt
import numpy as np
from pandas.tools.plotting import scatter_matrix
import statsmodels.api as sm
import statsmodels.formula.api as smf

with open("exp_dataset_p_interp.json") as json_data:
    d = json.load(json_data)

# # # # # # # # #
# Create Data Frame
trials_data = pd.DataFrame(d['trials'])
print trials_data.columns

# Creat Pupil Response variable
trials_data['pupil_response'] = (trials_data.max_diam - trials_data.trial_baseline) * 100 / trials_data.pre_exp_baseline
trials_data['pupil'] = (trials_data.max_diam - trials_data.trial_baseline) * 100 / trials_data.trial_baseline

# Create new Valence Rating
trials_data['valence_intensity'] = trials_data.valence_rating - 50.0  #np.absolute(trials_data.valence_rating - 50.0)
trials_data['valence_absolute'] = np.absolute(trials_data.valence_rating - 50.0)

# Create new Subjective Rating
trials_data['subjective_absolute'] = np.absolute(trials_data.subjective_rating - 50.0)

# Create pupil increase variable
trials_data['pupil_response_rate'] = trials_data.max_diam - trials_data.trial_baseline / trials_data.max_diam_time
#trials_data['pupil_response_rate'] = trials_data.pupil_response / trials_data.max_diam_time

gb = trials_data.groupby(('ambiguity', 'sound_valence'))

gb.arrow_response_time.describe()

# # # # # # # # #
# Basic data exclusion: too long responses and too much data loss
trials_data = trials_data.loc[(trials_data.arrow_response_time <= 9) & (trials_data.arrow_response_time > .2) & (trials_data.data_loss < 20)]


# # # # # # # # # # # # #
# Sound Parameters Data Set
sound_info = pd.read_csv("sound_params.csv")

# ['duration', 'validation_propCorVal', 'validation_entropy', 'validation_maxEmIntensity']
sound_info = sound_info.loc[:, 'duration']


# Concatenate Experiment Data and Sound Parameters
#full_data = pd.concat([trials_data, sound_info], axis = 1)
#print full_data.head(12)


# # # # # # # # # # # # # #
# Save DataSet
if True:
    trials_data.to_csv('spd_dataset_interp.csv') # "spd_full_dataset"
