# -*- coding: utf-8 -*-
"""
@author: Manuel
Interpolates (linear) the pupil response and returns a 30Hz output.
This donwsamples the trial and smooths blinks.
"""

import pandas as pd
import numpy as np
import scipy.signal
import json
import matplotlib.pyplot as plt
from scipy import interpolate


filepath = '/Users/molweis/Documents/SPD_Data/1492506976.txt'

data = pd.read_csv(filepath, sep='\t')

col_names = ['Time', 'Type', 'L Raw X [px]', 'R Raw X [px]', 'L Pupil Diameter [mm]', 'R Pupil Diameter [mm]']

data = data.loc[:, col_names]

data.columns = ('time','type', 'lx', 'rx', 'ldiam', 'rdiam')

msg = data.loc[data.type == 'MSG']
smp = data.loc[data.type == 'SMP']

msg.lx = msg.lx.apply(lambda x: x[len('# Message: '):])

data = pd.concat((msg, smp)).sort_index()


# # # # # # # # # # # # # # #
def remove_blinks(trial_df):

    diam = np.array(trial_df.rdiam)
    time = np.array(trial_df.time)
    print 'duration: ', time[-1]
    rx = np.array(trial_df.rx)

    # fill message nans
    nans = np.isnan(diam)
    diam[nans] = np.interp(time[nans], time[~nans], diam[~nans])

    # Blinks into nans
    diam[diam < 1] = np.nan
    rx[diam < 1] = np.nan

    nan_sum = np.sum(np.isnan(diam))

    # plt.plot(diam)
    # plt.show()

    # Calculate % of missing data
    missing_data = nan_sum * 100.0/len(diam)
    print 'Missing Data: %.5f' % missing_data

    # Only smooth if there is enough data
    if missing_data < 75:
        diam_copy = list(diam)

        for i in xrange(len(diam)):
            if np.isnan(diam_copy[i]) == True:
                sample_range = 9
                diam[i-sample_range:i] = np.nan
                diam[i:i+sample_range] = np.nan

                rx[i-sample_range:i] = np.nan
                rx[i:i+sample_range] = np.nan

        nans = np.isnan(diam)

        diam[nans] = np.interp(time[nans], time[~nans], diam[~nans])
        #diam = scipy.signal.savgol_filter(diam, 21, 3)

        if True:
            print "Create Interpolation Function"
            f = interpolate.interp1d(time, diam, 'slinear')

            frec = time[-1] * 30 # set frequency for interpolation 30Hz

            time = np.linspace(0, time[-1], frec)

            diam = f(time)

        diam = scipy.signal.savgol_filter(diam, 21, 3)

    return diam, time, missing_data, trial_df, rx



#########################
Pre Experiment Baseline

start_idx = data.loc[data.lx == 'pre_experiment_baseline_start'].index[0]
end_idx = data.loc[data.lx == 'pre_experiment_baseline_end'].index[0]

pre_base = data.loc[start_idx:end_idx, ('time', 'ldiam', 'rdiam', 'rx')]
pre_base.time = pre_base.time / 10 ** 6
pre_base.time = pre_base.time - pre_base.time.iloc[0]

diam, time, data_loss, trial_df, rx = remove_blinks(pre_base)

#plt.plot(time, diam, '-x')

time_interp = range(0,len(diam))
print time_interp
plt.plot(time_interp, diam, '-o')

# raw_diam = pre_base.rdiam.values
# interp_func = interpolate.
#plt.plot(pre_base.rdiam, color='blue', alpha=0.3)

plt.axhline(np.mean(diam))

plt.title('Pre-experiment Baseline')
plt.show()


####
# Open JSON, check length, use length to set range in for loop, extract et data,
json_filepath = filepath = '/Users/molweis/Documents/SPD_Data/1493212568.json'
with open(json_filepath) as json_data:
    d = json.load(json_data)

json_length = len(d['experiment_data'])

########
# Trial
for trial_n in xrange(1, json_length):
    print "Trial #: ", trial_n

    # Json
    json_trial = d['experiment_data'][trial_n-1]

    # Get idxs
    trial_start = 'trial_start:' + str(trial_n)
    trial_end = 'trial_end:' + str(trial_n)
    trial_finish = 'trial_finished:' + str(trial_n)

    # Set trial Start and End indexes
    start_idx = data.loc[data.lx == trial_start].index[0]
    end_idx = data.loc[data.lx == trial_end].index[0]

    # Keep relevant columns, make time relative to trial start
    trial = data.loc[start_idx: end_idx, ('time', 'rdiam', 'ldiam', 'lx', 'rx', 'type')]
    trial.time = trial.time/10**6
    trial.time = trial.time - trial.time.iloc[0]

    # trial messages
    trial_msgs = trial.loc[trial.type == 'MSG']

    sound_start_time = trial_msgs.time.iloc[1]

    sound_file = trial_msgs.lx.iloc[1]

    response_time = trial_msgs.time.iloc[2]

    # Pre-process eye data
    diam, time, data_loss, trial_df, rx = remove_blinks(trial)

    # # Get baseline
    baseline_time_start = sound_start_time - 1.3
    trial_baseline = diam[(time < sound_start_time) & (time > sound_start_time - 1.3)].mean()

    # Get maximum response
    max_diam = np.nanmax(diam[time > sound_start_time])
    max_diam_time = time[diam == max_diam]

    # if np.isnan(max_diam):
    #     print "Eureka"
    #     max_diam = np.nan
    #     max_diam_time = [np.nan]

    if len(max_diam_time) == 0:
        max_diam_time = -1

    elif len(max_diam_time) > 1:
        max_diam_time = max_diam_time[0]


    print "Base, RT, Max, Max-time: ", trial_baseline, response_time, max_diam, float(max_diam_time)

    print "Sampling diff (mean +/- std) :", np.mean(np.diff(time)), np.std(np.diff(time))

    select_idx = time > sound_start_time - 1.3
    time = time[select_idx]
    diam = diam[select_idx]


    plt.plot(time, diam, '-x')
    plt.axvline(sound_start_time, color='r')
    plt.axvline(response_time, color='g')
    plt.axvline(max_diam_time, color='b')
    plt.axvline(baseline_time_start, color='y')
    plt.axhline(trial_baseline)
    plt.axhline(max_diam)
    plt.title('Pupil Response')
    plt.xlabel('Time (s)')
    plt.ylabel('Pupil diameter (mm)')
    plt.show()
