# -*- coding: utf-8 -*-
"""
@author: Manuel
§Grabs txt and json files from individual participants
§Removes Blinks (grabs functions from pupil_response.py)
§Checks missing data, preprocess good trials and ignores when too much data loss (>25%)
§Output is used for "creates_full_dataset.py (this one excludes trial based on data loss
calculated here)
"""

import pandas as pd
import numpy as np
import scipy.signal
import json
import matplotlib.pyplot as plt
from scipy import interpolate
import glob

filepath = "/Users/molweis/Documents/SPD_Data/"
text_file_path = filepath + '*.txt'
json_file_path = filepath + '*.json'
text_file_list = glob.glob(text_file_path)
json_file_list = glob.glob(json_file_path)
zipped_file_list = zip(text_file_list, json_file_list)

# # # # # # # # # # # # #
def remove_blinks(trial_df):

    diam = np.array(trial_df.rdiam)
    time = np.array(trial_df.time)

    # fill message nans
    nans = np.isnan(diam)
    diam[nans] = np.interp(time[nans], time[~nans], diam[~nans])

    # Blinks into nans
    diam[diam < 1] = np.nan

    nan_sum = np.sum(np.isnan(diam))

    # plt.plot(diam)
    # plt.show()

    # Calculate % of missing data
    missing_data = nan_sum * 100.0/len(diam)
    #print 'Missing Data: %.5f' % missing_data

    # Only smooth if there is enough data
    if missing_data < 75:
        diam_copy = list(diam)

        for i in xrange(len(diam)):
            if np.isnan(diam_copy[i]) == True:
                sample_range = 9
                diam[i-sample_range:i] = np.nan
                diam[i:i+sample_range] = np.nan

        nans = np.isnan(diam)

        diam[nans] = np.interp(time[nans], time[~nans], diam[~nans])
        #diam = scipy.signal.savgol_filter(diam, 21, 3)

        if True:
            #print "Create Interpolation Function"
            f = interpolate.interp1d(time, diam, 'slinear')

            frec = time[-1] * 30 # set frequency for interpolation 30Hz

            time = np.linspace(0, time[-1], frec)

            diam = f(time)

        diam = scipy.signal.savgol_filter(diam, 21, 3)

    return diam, time, missing_data

##############
all_trials = []

for experiment_file in zipped_file_list:
    experiment_file[0] # txt file
    experiment_file[1] # json file

    filepath = str(experiment_file[0])
    pid = filepath[-14:-4]
    print 'PID: ', pid

    data = pd.read_csv(filepath, sep='\t')

    # keep relevant columns
    col_names = ['Time', 'Type', 'L Raw X [px]', 'L Pupil Diameter [mm]', 'R Pupil Diameter [mm]']
    data = data.loc[:, col_names]

    # rename columns
    data.columns = ('time','type', 'lx', 'ldiam', 'rdiam')

    msg = data.loc[data.type == 'MSG']
    smp = data.loc[data.type == 'SMP']

    # Shorten messagges
    msg.lx = msg.lx.apply(lambda x: x[len('# Message: '):])

    # Concat again
    data = pd.concat((msg, smp)).sort_index()


    #########################
    # Pre Experiment Baseline

    start_idx = data.loc[data.lx == 'pre_experiment_baseline_start'].index[0]
    end_idx = data.loc[data.lx == 'pre_experiment_baseline_end'].index[0]

    pre_base = data.loc[start_idx:end_idx, ('time', 'ldiam', 'rdiam')]
    pre_base.time = pre_base.time / 10 ** 6
    pre_base.time = pre_base.time - pre_base.time.iloc[0]

    diam, time, data_loss = remove_blinks(pre_base)

    pre_exp_baseline = np.nanmean(diam)

    if False:
        plt.figure()
        plt.plot(time,diam, color='red')
        plt.plot(pre_base.time, pre_base.rdiam, color='blue', alpha=0.4)
        plt.axhline(pre_exp_baseline)

        plt.title('Pre-experiment Baseline')
        plt.show()

    # # # # # # # #
    # Open JSON, check length, use length to set range in for loop, extract et data,
    json_filepath = experiment_file[1]
    with open(json_filepath) as json_data:
        d = json.load(json_data)

    json_length = len(d['experiment_data'])

    final_questions = d['final_questions']

    params = d['params']
    print params

    # # # # #
    # Go to participant trials

    for trial_n in xrange(1, json_length+1):

        # JSON: get the json trial. This already contains experimental data
        json_trial = d['experiment_data'][trial_n-1] # -1 because this on starts from 0

        # Get idxs
        trial_start = 'trial_start:' + str(trial_n)
        trial_end = 'trial_end:' + str(trial_n)
        trial_finish = 'trial_finished:' + str(trial_n)

        start_idx = data.loc[data.lx == trial_start].index[0]
        end_idx = data.loc[data.lx == trial_end].index[0]

        # Keep relevant columns, make time relative to trial start
        trial = data.loc[start_idx: end_idx, ('time', 'rdiam', 'ldiam', 'lx', 'type')]
        trial.time = trial.time/10**6
        trial.time = trial.time - trial.time.iloc[0] # convert to relative time

        # trial messages
        trial_msgs = trial.loc[trial.type == 'MSG']

        sound_start_time = trial_msgs.time.iloc[1]

        sound_file = trial_msgs.lx.iloc[1]

        response_time = trial_msgs.time.iloc[2]

        # Pre-process eye data
        diam, time, data_loss = remove_blinks(trial)

        # Get baseline
        baseline_size = 1.3
        trial_baseline =  diam[(time < sound_start_time) & (time > sound_start_time - baseline_size)].mean()

        # Get maximum response
        max_diam = np.nanmax(diam[time > sound_start_time])
        max_diam_time = time[diam==max_diam] - sound_start_time # time since the start of the sound

        # Get pupil samples for the trial
        trial_samples = diam[(time > sound_start_time - baseline_size)]

        # Classify sound
        sound_file = json_trial['sound_file']
        if sound_file.find('sex') > 0:
            json_trial['erotic'] = 1
        if sound_file.find('sex') < 0:
            json_trial['erotic'] = 0


        split_sound_file = sound_file.split('\\')
        json_trial['ambiguity'] = split_sound_file[1][:3]
        json_trial['sound_valence'] = split_sound_file[2][:3]
        json_trial['sound_file_name'] = split_sound_file[2][4:]

        json_trial['trial_baseline'] = trial_baseline
        json_trial['max_diam'] = float(max_diam)
        json_trial['trial_samples'] = trial_samples.tolist()

        # If no max is found
        if len(max_diam_time) == 0:
            max_diam_time = -1

        # If two max are found
        elif len(max_diam_time) > 1:
            max_diam_time = max_diam_time[0]



        json_trial['max_diam_time'] = float(max_diam_time)
        json_trial['data_loss'] = data_loss
        json_trial['pre_exp_baseline'] = pre_exp_baseline
        json_trial['pid'] = pid

        json_trial['social_distraction_rating'] = final_questions['social_distraction_rating']
        json_trial['social_influence_rating'] = final_questions['social_influence_rating']
        json_trial['extro_intro_rating'] = final_questions['extro_intro_rating']
        json_trial['social_rating'] = final_questions['social_rating']

        json_trial['gender'] = params['gender']
        json_trial['condition'] = params['ExpVersion']
        json_trial['age'] = params['Age']
        json_trial['session_size'] = params['N']


        #print json_trial

        all_trials.append(json_trial)

preprocessed_data = {'trials': all_trials}

# Save preprocessed data set:
save_as = str(pid) + '_pro' + '.json'
save_as = 'exp_dataset_p_interp.json'

if False:
    with open(save_as, 'w') as outfile:
        json.dump(preprocessed_data, outfile)
