# -*- coding: utf-8 -*-
'''
@author: Manuel
'''
import pandas as pd
import numpy as np
import scipy.signal
import json
import matplotlib.pyplot as plt
from scipy.interpolate import splev, splrep

filepath = '/Users/molweis/Documents/SPD_Data/1492506976.txt'

data = pd.read_csv(filepath, sep='\t')

col_names = ['Time', 'Type', 'L Raw X [px]', 'R Raw X [px]', 'L Pupil Diameter [mm]', 'R Pupil Diameter [mm]']

data = data.loc[:, col_names]

data.columns = ('time','type', 'lx', 'rx', 'ldiam', 'rdiam')

msg = data.loc[data.type == 'MSG']
smp = data.loc[data.type == 'SMP']

msg.lx = msg.lx.apply(lambda x: x[len('# Message: '):])

data = pd.concat((msg, smp)).sort_index()


# # # # # # # # # # # # # # #
def remove_blinks(trial_df):

    diam = np.array(trial_df.rdiam)
    time = np.array(trial_df.time)
    rx = np.array(trial_df.rx)

    # fill message nans
    nans = np.isnan(diam)
    diam[nans] = np.interp(time[nans], time[~nans], diam[~nans])

    # Blinks into nans
    diam[diam < 1] = np.nan
    rx[diam < 1] = np.nan

    nan_sum = np.sum(np.isnan(diam))

    # plt.plot(diam)
    # plt.show()

    # Calculate % of missing data
    missing_data = nan_sum * 100.0/len(diam)
    print 'Missing Data: %.5f' % missing_data

    # Only smooth if there is enough data
    if missing_data < 75:
        diam_copy = list(diam)

        for i in xrange(len(diam)):
            if np.isnan(diam_copy[i]) == True:
                sample_range = 9
                diam[i-sample_range:i] = np.nan
                diam[i:i+sample_range] = np.nan

                rx[i-sample_range:i] = np.nan
                rx[i:i+sample_range] = np.nan

        nans = np.isnan(diam)


        diam[nans] = np.interp(time[nans], time[~nans], diam[~nans])
        diam = scipy.signal.savgol_filter(diam, 21, 3)


    # nans_rx = np.isnan(rx)
    # rx[nans_rx] = np.interp(time[nans_rx], time[~nans_rx], rx[~nans_rx])
    # rx = scipy.signal.savgol_filter(rx, 21, 3)

    trial_df['filtered_diam'] = diam
    return diam, time, missing_data, trial_df, rx



#########################
# Pre Experiment Baseline

start_idx = data.loc[data.lx == 'pre_experiment_baseline_start'].index[0]
end_idx = data.loc[data.lx == 'pre_experiment_baseline_end'].index[0]

pre_base = data.loc[start_idx:end_idx, ('time', 'ldiam', 'rdiam', 'rx')]

pre_base['diam'] = (pre_base.ldiam + pre_base.rdiam)/2.0

diam, time, data_loss, trial_df, rx = remove_blinks(pre_base)

plt.figure()
plt.plot(time,diam, color='red')
#plt.plot(time,rx)
plt.plot(pre_base.time, pre_base.rdiam, color='blue', alpha=0.3)
plt.axhline(np.mean(diam))

plt.title('Pre-experiment Baseline')
plt.show()


####
# Open JSON, check length, use length to set range in for loop, extract et data,
json_filepath = filepath = '/Users/molweis/Documents/SPD_Data/1493212568.json'
with open(json_filepath) as json_data:
    d = json.load(json_data)

json_length = len(d['experiment_data'])

########
# Trial
for trial_n in xrange(1, json_length):
    print "Trial #: ", trial_n

    # Json
    json_trial = d['experiment_data'][trial_n-1]

    # Get idxs
    trial_start = 'trial_start:' + str(trial_n)
    trial_end = 'trial_end:' + str(trial_n)
    trial_finish = 'trial_finished:' + str(trial_n)

    # Set trial Start and End indexes
    start_idx = data.loc[data.lx == trial_start].index[0]
    end_idx = data.loc[data.lx == trial_end].index[0]

    # Keep relevant columns, make time relative to trial start
    trial = data.loc[start_idx: end_idx, ('time', 'rdiam', 'ldiam', 'lx', 'rx', 'type')]
    trial.time = trial.time/10**6
    trial.time = trial.time - trial.time.iloc[0]

    # trial messages
    trial_msgs = trial.loc[trial.type == 'MSG']

    sound_start_time = trial_msgs.time.iloc[1]

    sound_file = trial_msgs.lx.iloc[1]

    response_time = trial_msgs.time.iloc[2]

    # Pre-process eye data
    diam, time, data_loss, trial_df, rx = remove_blinks(trial)

    # Get baseline
    trial_baseline = diam[(time < sound_start_time) & (time > sound_start_time - 1.3)].mean()

    # Get maximum response
    max_diam = np.nanmax(diam[time > sound_start_time])
    max_diam_time = time[diam == max_diam]

    # if np.isnan(max_diam):
    #     print "Eureka"
    #     max_diam = np.nan
    #     max_diam_time = [np.nan]

    if len(max_diam_time) == 0:
        max_diam_time = -1

    elif len(max_diam_time) > 1:
        max_diam_time = max_diam_time[0]


    print trial_baseline, response_time, max_diam, float(max_diam_time)

    select_idx = time > sound_start_time - 1.0
    time = time[select_idx]
    diam = diam[select_idx]

    plt.plot(time, diam)
    plt.axvline(sound_start_time, color='r')
    plt.axvline(response_time, color='g')
    plt.axvline(max_diam_time, color='b')
    plt.axhline(trial_baseline)
    plt.axhline(max_diam)
    plt.show()






def detect_blinks(samples, blink_threshold=2):
    temp = np.gradient(samples.area)

    w = np.ones(100)
    w /= w.sum()

    temp = np.convolve(np.abs(temp), w, mode='same')

    return temp < blink_threshold

class Preprocessor(object):

    def __init__(self, et_filename, spec_filename):
        self.et_filename = et_filename
        self.spec_filename = spec_filename

        self._setup()
        self._preprocess_samples()

    def _setup(self):
        # Read dataframe with all data
        self.dataframe = pd.read_table(self.et_filename)

        # Split into samples and messages
        self.samples = self.dataframe[self.dataframe['Type'] == 'SMP']
        self.messages = self.dataframe[self.dataframe['Type'] == 'MSG']

        # Remove and rename columns
        self.samples = self.samples.loc[:, ('Time', 'R Raw X [px]', 'R Raw Y [px]', 'R Area [px]')]
        self.samples.reset_index(inplace=True, drop=True)
        self.samples.columns = ('time', 'x', 'y', 'area')
        self.messages = self.messages.loc[:, ('Time', 'R Raw X [px]')]
        self.messages.reset_index(inplace=True, drop=True)
        self.messages.columns = ('time', 'message')

        # Remove the '# Message: ' part of each message
        self.messages.message = self.messages.message.apply(lambda s: s[len('# Message: '):])

        # Specification JSON
        with open(self.spec_filename, 'r') as f:
            spec_data = json.load(f)

        spec_data['reaction_time'] = spec_data['rt']
        spec_data['trial_correct'] = [bool(c) for c in spec_data['trial_correct']]

        keys = ('load', 'target', 'distractor', 'congruency', 'response', 'trial_correct', 'reaction_time')
        spec_data = zip(*[spec_data[key] for key in keys])

        self.spec_data = [{key: value for key, value in zip(keys, values)} for values in spec_data]

    def _preprocess_samples(self):
        self.samples.area = scipy.signal.savgol_filter(self.samples.area, 51, 3)

    def get_samples_between(self, start_time, stop_time):
        return self.samples[(self.samples.time >= start_time) & (self.samples.time <= stop_time)].reset_index(drop=True)

    def get_sample_statistics(self, sample, stimulus_onset_time):
        baseline_start_time = int(stimulus_onset_time - 1000000)

        # Pupil area baseline (mean over 1 second before the stimulus onset)
        pupil_area_baseline = sample.area[(sample.time >= baseline_start_time) & (sample.time <= stimulus_onset_time)].mean()

        # Find maximum after stimulus onset
        max_idx = sample.area[sample.time >= stimulus_onset_time].argmax()
        pupil_area_max = sample.area.iloc[max_idx]
        pupil_area_max_time = sample.time.iloc[max_idx]

        # Check if there is a blink before the time of the maxmum pupil area
        blink_mask = detect_blinks(sample)
        time_mask = ((sample.time >= baseline_start_time) & (sample.time <= pupil_area_max_time + 10 * 1e3)).values
        valid = all(blink_mask[time_mask])

        return {
            'baseline': pupil_area_baseline,
            'max_area': pupil_area_max,
            'max_time': pupil_area_max_time,
            'valid': valid
        }

    def get_pre_experiment_baseline(self):
        start_time = self.messages.time[self.messages.message.str.startswith('start_pupil_baseline')].iloc[0]
        stop_time = self.messages.time[self.messages.message.str.startswith('end_pupil_baseline')].iloc[0]

        samples = self.get_samples_between(start_time, stop_time)

        blink_mask = detect_blinks(samples)

        baseline_area = samples.area[blink_mask].mean()

        return baseline_area

    def get_trial_blocks(self):
        spec_data = list(self.spec_data)

        block_start_times = self.messages.time[self.messages.message.str.startswith('block_start')]
        block_end_times = self.messages.time[self.messages.message.str.startswith('block_end')]

        blocks = []

        for start_time, stop_time in zip(block_start_times, block_end_times):
            block_messages = self.messages[(self.messages.time >= start_time) & (self.messages.time <= stop_time)]

            trial_onset_times = block_messages.time[block_messages.message.str.startswith('fixation_onset_trial')].values
            stimulus_onset_times = block_messages.time[block_messages.message.str.startswith('stimulus_onset_trial')].values
            stimulus_offset_times = block_messages.time[block_messages.message.str.startswith('stimulus_offset_trial')].values
            trial_offset_times = block_messages.time[block_messages.message.str.startswith('fixation_offset_trial')].values

            num_trials = min(len(trial_onset_times), len(trial_offset_times))
            trials = []

            for i in range(num_trials):
                trial_samples = self.get_samples_between(trial_onset_times[i], trial_offset_times[i])

                trial = {
                    'start_time': trial_onset_times[i],
                    'stop_time': trial_offset_times[i],
                    'stimulus_onset': stimulus_onset_times[i],
                    'stimulus_offset': stimulus_offset_times[i],
                    'samples': trial_samples
                }

                trial.update(self.get_sample_statistics(trial_samples, stimulus_onset_times[i]))
                trial.update(spec_data.pop(0))

                trials.append(trial)

            block = {
                'start_time': start_time,
                'stop_time': stop_time,
                'trials': trials
            }

            blocks.append(block)

        return blocks


if __name__ == '__main__':

    pid_list = [1488204821, 1488284821, 1489492974, 1489584012, 1489589534, 1489671183,
            1490360989, 1490771540, 1490775527, 1490779004, 1490793447,1490797371,
            1490861934, 1490865089, 1490869270, 1490872788, 1490879689]

    all_participants = []

    for pid in pid_list:

        print pid

        txt_file = str(pid) + '.txt'
        json_file = str(pid) + '.json'

        preprocessor = Preprocessor(
            '/Users/molweis/Documents/Data/' + txt_file,
            '/Users/molweis/Documents/Data/' + json_file)


        blocks = preprocessor.get_trial_blocks()

        trials = pd.DataFrame(sum([block['trials'] for block in blocks], []))

        trials['pid'] = int(pid)

        pre_exp_baseline = preprocessor.get_pre_experiment_baseline()
        trials['pre_baseline'] = pre_exp_baseline

        trials['max_area'] = trials.max_area / pre_exp_baseline
        trials['baseline'] = trials.baseline / pre_exp_baseline
        trials['pupil_dilation'] = (trials.max_area - trials.baseline) / trials.baseline

        gb = trials.groupby(('load', 'congruency'))
        keys =  list(gb.groups.keys())
        key_strings = map(lambda k: '{}_{}'.format(*k), keys)

        reaction_time = list(gb.reaction_time.mean())

        all_participants.append(trials)

    pdpl_data = pd.concat(all_participants, axis=0)
    pdpl_data.to_csv('pdpl_data_normalized.csv')

    exit()

if __name__ == '__main__':
    pass

    preprocessor = Preprocessor(
        '/Users/molweis/Documents/Data/1488204821.txt',
        '/Users/molweis/Documents/Data/1488204821.json'
    )

    blocks = preprocessor.get_trial_blocks()

    import matplotlib.pyplot as plt

    trials = pd.DataFrame(sum([block['trials'] for block in blocks], []))

    trials = trials[(trials.trial_correct == True) & (trials.valid == True)]
    #trials = trials[trials.load == 'L']

    baseline = preprocessor.get_pre_experiment_baseline()

    trials['max_area'] = trials.max_area / baseline
    trials['baseline'] = trials.baseline / baseline
    trials['pupil_dilation'] = (trials.max_area - trials.baseline) / trials.baseline

    grouped_trials = trials.groupby(('congruency'))
    means = grouped_trials.mean()

    import matplotlib.pyplot as plt

    trial = trials.iloc[4, :]

    print trial.max_area

    plt.plot(trial.samples.time/1e6, trial.samples.area)

    plt.vlines(trial.stimulus_onset/1e6, trial.baseline, trial.max_area)
    plt.hlines(float(trial.baseline), (trial.stimulus_onset-1e6)/1e6, trial.stimulus_onset/1e6)
    plt.show()

    # plt.subplot(121)
    # plt.hist(grouped_trials.get_group('congruent').baseline.values, 50)
    # plt.title('Congruent')
    # plt.xlim(200, 1100)
    # plt.subplot(122)
    # plt.hist(grouped_trials.get_group('incongruent').baseline.values, 50)
    # plt.title('Incongruent')
    # plt.xlim(200, 1100)
    # plt.show()


    labels = [str(key) for key in grouped_trials.groups.keys()]

    # plt.figure()
#     plt.bar(1 + np.arange(2), means.baseline, width=0.8, yerr=stds.baseline, tick_label=labels, align='center')
#     plt.ylabel('pupil area baseline (px)')
#     plt.show()

    plt.figure()
    plt.scatter(grouped_trials.get_group('congruent').baseline.values, grouped_trials.get_group('congruent').reaction_time.values, color='blue')
    plt.scatter(grouped_trials.get_group('incongruent').baseline.values, grouped_trials.get_group('incongruent').reaction_time.values, color='red')
    plt.title('reaction time vs baseline')
    plt.show()

    plt.figure()
    plt.scatter(grouped_trials.get_group('congruent').max_area.values, grouped_trials.get_group('congruent').reaction_time.values, color='blue')
    plt.scatter(grouped_trials.get_group('incongruent').max_area.values, grouped_trials.get_group('incongruent').reaction_time.values, color='red')
    plt.title('reaction time vs max dilation')
    plt.show()

    plt.figure()
    plt.scatter(grouped_trials.get_group('congruent').pupil_dilation.values, grouped_trials.get_group('congruent').reaction_time.values, color='blue')
    plt.scatter(grouped_trials.get_group('incongruent').pupil_dilation.values, grouped_trials.get_group('incongruent').reaction_time.values, color='red')
    plt.title('reaction time vs pupil dilation')
    plt.show()

    import statsmodels.formula.api as smf
    from statsmodels.stats.anova import anova_lm

    # trials = trials[trials.load == 'L']



    exit()

if __name__ == '__main__':
    preprocessor = Preprocessor(
        '/Users/molweis/Documents/Data/1488284821.txt',
        '/Users/molweis/Documents/Data/1488284821.json'
    )

    blocks = preprocessor.get_trial_blocks()

    import matplotlib.pyplot as plt

    trials = pd.DataFrame(sum([block['trials'] for block in blocks], []))
    trials = trials[trials.trial_correct == True]
    trials.load[trials.load == ' '] = 'H'

    trials['pupil_dilation'] = (trials.max_area - trials.baseline) / trials.baseline

    grouped_trials = trials.groupby(('load', 'congruency'))
    means = grouped_trials.mean()
    stds = grouped_trials.std()

    import matplotlib.pyplot as plt
    plt.style.use('ggplot')

    labels = ['{} {}'.format(*key) for key in grouped_trials.groups.keys()]

    plt.figure()
    plt.bar(1 + np.arange(4), 100*means.pupil_dilation , width=0.8, yerr=100*stds.pupil_dilation, tick_label=labels, align='center')
    plt.ylabel('Pupil Dilation (%)')

    plt.figure()
    plt.bar(1 + np.arange(4), means.baseline, width=0.8, yerr=stds.baseline, tick_label=labels, align='center')
    plt.ylabel('Baseline')

    plt.figure()
    plt.bar(1 + np.arange(4), means.max_area, width=0.8, yerr=stds.max_area, tick_label=labels, align='center')
    plt.ylabel('Maximum Dilation')

    plt.show()
    exit()


if __name__ == '__main__':
    preprocessor = Preprocessor(
        '/Users/molweis/Documents/Data/1489499584.txt',
        '/Users/molweis/Documents/Data/1489499584.json'
    )

    blocks = preprocessor.get_trial_blocks()

    import matplotlib.pyplot as plt

    trials = pd.DataFrame(sum([block['trials'] for block in blocks[3:]], []))

    for i, congruency in enumerate(['congruent', 'incongruent']):
        c_trials = trials[(trials.trial_correct == True) & (trials.congruency == congruency)]

        pupil_dilation = (c_trials.max_area - c_trials.baseline) / c_trials.baseline

        print u'{}: {} +/- {}'.format(congruency, pupil_dilation.mean(), pupil_dilation.std())

        plt.subplot(1, 2, i + 1)
        plt.plot(pupil_dilation, c_trials.reaction_time, '.')
        plt.title(congruency.capitalize())

    plt.show()
