## Perceived and induced emotion, confidence, and duration per emotion

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/Manuel_pupil-response/Analysis')

library (lme4)
library (ggplot2)
library (brms)

df = read.csv('spd_dataset_interp.csv')[,-1]
colnames(df)

## Perceived vs induced emotion ratings
plot (df$valence_rating, df$subjective_rating, main=paste0('r = ', round(cor(df$valence_rating, df$subjective_rating),2))) # highly correlated
plot (df$valence_absolute, df$subjective_absolute, main=paste0('r = ', round(cor(df$valence_absolute, df$subjective_absolute),2))) 

plot (df$valence_rating, df$pupil_response) # non-linear


# which is a better predictor of pupil_response: perceived or induced emotion?
mod1 = lmer(pupil_response~valence_absolute+(1|pid)+(1|sound_file), df) # BEST MODEL
summary(mod1)
drop1(mod1, test='Chisq')

mod2 = lmer(pupil_response~subjective_absolute+(1|pid)+(1|sound_file), df) 
summary(mod2)
drop1(mod2, test='Chisq')

mod3 = lmer(pupil_response~valence_absolute+subjective_absolute+(1|pid)+(1|sound_file), df) 
summary(mod3)
drop1(mod3, test='Chisq')

AIC (mod1, mod2, mod3)




# same but based on auc
mod1 = lmer(auc~valence_absolute+(1|pid)+(1|sound_file), df) 
summary(mod1)
drop1(mod1, test='Chisq') # non-signif.

mod2 = lmer(auc~subjective_absolute+(1|pid)+(1|sound_file), df) 
summary(mod2)
drop1(mod2, test='Chisq') # non-signif.

# same but based on time_pupil_response
mod1 = lmer(time_pupil_response~valence_absolute+(1|pid)+(1|sound_file), df) 
summary(mod1)
drop1(mod1, test='Chisq') # L = 7.7, df = 1, p = .005

mod2 = lmer(time_pupil_response~subjective_absolute+(1|pid)+(1|sound_file), df) 
summary(mod2)
drop1(mod2, test='Chisq') # non-signif.




### Perceived / induced intensity ratings per emotion category
mod = lmer(valence_rating/100~emotion+(1|pid)+(1|sound_file), data=df)
drop1(mod, test='Chisq')
plot (mod)

min(df$valence_rating[df$valence_rating>0])
max(df$valence_rating[df$valence_rating<100]) # 1% and 99% are the closest values to 0 and 1. We transform by replacing zero with 0.5 and 100 with 99.5
df$valence_rating[df$valence_rating==0] = 0.5
df$valence_rating[df$valence_rating==100] = 99.5
df$vr_logit = log (df$valence_rating / (100-df$valence_rating))
hist (df$vr_logit)

mod1 = lmer(vr_logit~emotion+(1|pid)+(1|sound_file), data=df)
drop1(mod1, test='Chisq')
plot (mod1)

mod2 = glmer (valence_rating/100~emotion+(1|pid)+(1|sound_file), data=df, family=gaussian(link='logit'))
plot (mod2)
summary (mod2)

AIC(mod, mod2)

# does everyone use the entire scale?
aggregate (valence_rating~pid, df, range) # yes

# preparing data for modeling
hist (df$valence_rating)
temp1 = df[,c('emotion','sound_file','pid','valence_rating','duration')]
temp1$measure = 'perceived'
colnames(temp1)[4] = 'rating'
temp2 = df[,c('emotion','sound_file','pid','subjective_rating','duration')]
temp2$measure = 'induced'
colnames(temp2)[4] = 'rating'
df_melt = rbind (temp1, temp2)
df_melt$rating = (df_melt$rating+1)/102
range (df_melt$rating)


# brms model
mod_brm_emRatings_per_emCat = brm (rating~emotion*measure+(1|pid)+(1|sound_file), data=df_melt, family='beta', cores=4, chains=4, iter=2000, prior=set_prior('normal(0,5)'))
# saveRDS(mod_brm_emRatings_per_emCat, 'mod_brm_emRatings_per_emCat.RDS') 
# mod_brm_emRatings_per_emCat = readRDS('mod_brm_emRatings_per_emCat.RDS')
summary (mod_brm_emRatings_per_emCat)
# plot (mod_brm_emRatings_per_emCat)
# pp_check(mod_brm_emRatings_per_emCat, nsamples = 30)

# plot fitted values
newdata = expand.grid(emotion=levels(df$emotion),
                      measure=c('perceived','induced'))
df_plot = cbind (newdata, fitted (mod_brm_emRatings_per_emCat, newdata, re_formula=NA))
colnames(df_plot) = c('emotion','measure','fit','se','lwr','upr')
df_plot[,3:6] = df_plot[,3:6]*102-1 # convert back to 0-100%
df_plot[,3:6] = df_plot[,3:6] - 50 # from 0...100 to -50...+50
df_plot$emotion = factor(df_plot$emotion, levels=c('amusement', 'joy', 'pleasure', 'neutral', 'disgust', 'fear', 'pain', 'sadness'))
df_melt$measure = factor(df_melt$measure, levels=c('perceived','induced'))

# plot
p1 = ggplot (df_plot, aes(x=emotion, y=fit, ymin=lwr, ymax=upr, shape=measure)) +
  geom_violin (data=df_melt, aes(x=emotion, y=rating*100-50, fill=measure), inherit.aes = F, alpha=.7, col='white', position=position_dodge(.7), show.legend = F) +
  geom_point(position=position_dodge(.7), size=3) +
  geom_errorbar(position=position_dodge(.7), width=0) +
  geom_vline(xintercept=3.5, linetype=2) + geom_vline(xintercept=4.5, linetype=2) +
  geom_hline(yintercept=0, linetype=2) +
  xlab('') + ylab('%') +
  scale_x_discrete(labels=c('amusement\nn = 6','joy\nn = 11','pleasure\nn = 15','neutral\nn = 8','disgust\nn = 9','fear\nn = 12','pain\nn = 3','sadness\nn = 8')) +
  scale_y_continuous(breaks=seq(-50, 50, by=10)) +
  scale_color_discrete(name='') +
  scale_shape_discrete(name='') +
  coord_flip(ylim=c(-50, 50), expand=F) +
  ggtitle('Valence') +
  theme_bw() +
  theme(plot.title = element_text(hjust = 0.5),
        legend.title = element_text(size = 0),
        legend.background = element_rect(fill = 'gray95'),
        legend.position = c(.75, .92),
        panel.grid = element_blank())
print(p1)
# ggsave('pix/perceivedInduced_per_emCat.jpg', width=8, height=15, units='cm', scale=1, dpi=300)


# were ratings of perceived emotion overall higher than for induced emotion?
newdata = expand.grid(measure=c('perceived','induced'), emotion=levels(df$emotion))
f = fitted (mod_brm_emRatings_per_emCat, newdata, re_formula = NA, summary=F)
f = abs(f*100-50)
for (i in 1:length(levels(df$emotion))){
  em = levels(df$emotion)[i]
  q = quantile(f[, i*2-1] - f[, i*2], probs=c(.5, .025, .975)) # perceived - induced
  q = round(q, 1)
  q = paste0(q[1], ' [', q[2], ', ', q[3], ']')
  print (paste(em, q))
}

### confidence
df$cf01 = (df$confidence_rating+1)/102

# brms model
mod_brm_confidence_per_emCat = brm (cf01~emotion+(1|pid)+(1|sound_file), data=df, family='beta', cores=4, chains=4, iter=2000, prior=set_prior('normal(0,5)'))
# saveRDS(mod_brm_confidence_per_emCat, 'mod_brm_confidence_per_emCat.RDS')
# mod_brm_confidence_per_emCat = readRDS('mod_brm_confidence_per_emCat.RDS')
summary (mod_brm_confidence_per_emCat)

newdata = data.frame(emotion=levels(df$emotion))
df_plot = cbind (newdata, fitted (mod_brm_confidence_per_emCat, newdata, re_formula=NA))
colnames(df_plot) = c('emotion','fit','se','lwr','upr')
df_plot[,2:5] = df_plot[,2:5]*102-1 # convert back to 0-100%
df$emotion = factor (df$emotion, levels=c('amusement', 'joy', 'pleasure', 'neutral', 'disgust', 'fear', 'pain', 'sadness'))
df_melt$emotion = factor (df_melt$emotion, levels=c('amusement', 'joy', 'pleasure', 'neutral', 'disgust', 'fear', 'pain', 'sadness'))

# plot
p2 = ggplot (df_plot, aes(x=emotion, y=fit, ymin=lwr, ymax=upr)) +
  geom_violin (data=df, aes(x=emotion, y=confidence_rating), inherit.aes = F, alpha=.7, col='white', fill='gray70') +
  geom_point(size=3) +
  geom_errorbar(width=0) +
  geom_vline(xintercept=3.5, linetype=2) + geom_vline(xintercept=4.5, linetype=2) +
  geom_hline(yintercept=50, linetype=2) +
  xlab('') + ylab('%') +
  scale_x_discrete(labels=c('amusement\nn = 6','joy\nn = 11','pleasure\nn = 15','neutral\nn = 8','disgust\nn = 9','fear\nn = 12','pain\nn = 3','sadness\nn = 8'), position='top') +
  # scale_x_discrete(labels=NULL, position='top') +
  scale_y_continuous(breaks=seq(0,100,by=10)) +
  coord_flip(ylim=c(0,100), expand=F) +
  ggtitle('Confidence') +
  theme_bw() +
  theme(plot.title = element_text(hjust = 0.5),
        legend.position = 'none', 
        panel.grid = element_blank(),
        axis.ticks = element_blank())
print(p2)
# ggsave('pix/confidence_per_emCat.jpg', width=8, height=13, units='cm', scale=1, dpi=300)

# add a duration plot
a = aggregate (duration~emotion, df, min)
a = rbind (a, aggregate (duration~emotion, df, median))
a = rbind (a, aggregate (duration~emotion, df, max))
a$stat = 'range'
a$stat [9:16] = 'med'
a$emotion = factor(a$emotion, levels=c('amusement', 'joy', 'pleasure', 'neutral', 'disgust', 'fear', 'pain', 'sadness'))

p3 = ggplot(a, aes(x=emotion, y=duration, shape=stat)) + 
  geom_line () +
  geom_point (size=2, show.legend = F) +
  scale_shape_manual(values=c(16, NA)) +
  geom_vline(xintercept=3.5, linetype=2) + geom_vline(xintercept=4.5, linetype=2) +
  # geom_hline(yintercept = 1.5, linetype=2, col='gray50') +
  # geom_hline(yintercept = 2.5, linetype=2, col='gray50') +
  xlab('') +
  ylab('s') +
  scale_x_discrete(labels=c('amusement\nn = 6','joy\nn = 11','pleasure\nn = 15','neutral\nn = 8','disgust\nn = 9','fear\nn = 12','pain\nn = 3','sadness\nn = 8'), position='top') +
  scale_y_continuous(breaks=1:6) +
  coord_flip() + 
  ggtitle('Duration') +
  theme_bw() + 
  theme(plot.title = element_text(hjust = 0.5),
        panel.grid = element_blank())
print(p3)

source('/home/allgoodguys/Documents/Soft_comp/R/ggplot_multiple.R') # http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_(ggplot2)/
multiplot(p1, p2, p3, cols=3)
# ggsave(plot = multiplot(p1, p2, p3, cols=3), filename = 'pix/descriptives.jpg', width=25, height=15, units='cm', scale=1, dpi=300)
