## What predicts the height of peak pupil dilation?

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/006_with-Manuel/analysis')

library(lme4)
library(ggplot2)
library(reshape2)
library(plot3D)
library(soundgen)


df = read.csv('spd_dataset_interp.csv')[,-1]
# colnames (df)
dur_limits = c(0, Inf) # dur_limits = c(1.5, 2.5)
df = df[which(df$duration > dur_limits[1] & 
                df$duration < dur_limits[2]), ]

# check bivariate relationships for non-linearity
df$vr_bin = cut(df$valence_rating, breaks=25)
df$rt_bin = cut(df$arrow_response_time, breaks=25)
df$duration_bin = cut(df$duration, breaks=25)
df$cf_bin = cut(df$confidence_rating, breaks=10)

peak_dr = aggregate (max_diam ~ duration_bin, df, mean)
plot (peak_dr)

peak_rt = aggregate (max_diam ~ rt_bin, df, mean)
plot (peak_rt)

peak_cf = aggregate (max_diam ~ cf_bin, df, mean)
plot (peak_cf)

peak_vr = aggregate (max_diam ~ vr_bin, df, mean)
plot (peak_vr) # the only clearly non-linear one

rt_dur = aggregate (duration ~ rt_bin, df, mean)
plot (rt_dur, xlab='Arrow response time (aggreg.), s', ylab='Sound duration, s')

## do we need to look separately at positive/negative sounds?
df$valence_rating_bin = cut(df$valence_rating, breaks=c(-0.1,40,60,100))
table(df$valence_rating_bin)
aggregate (max_diam~valence_rating_bin, df, mean)
aggregate (max_diam~valence_rating_bin, df, sd)

df_notNeutral = droplevels ( df[df$valence_rating_bin != "(40,60]", ] )
table(df_notNeutral$valence_rating_bin)

mod_posNeg = lmer(max_diam~valence_rating_bin+trial_baseline+(1|pid)+(1|sound_file), df_notNeutral) 
summary(mod_posNeg)
drop1(mod_posNeg, test='Chisq') # L = 2.1, df = 1, p = .14

mod_posNeg_dur = lmer(max_diam~valence_rating_bin+duration+trial_baseline+(1|pid)+(1|sound_file), df_notNeutral) 
summary(mod_posNeg_dur)
drop1(mod_posNeg_dur, test='Chisq') # L = 0.5, df = 1, p = .47

# plot
df$valence_rating_bin = cut(df$valence_rating, breaks=20)
plot (aggregate (max_diam~valence_rating_bin, df, mean))

# another approach: look at arrow response
mod_arrow = lmer(max_diam~arrow_response+trial_baseline+(1|pid)+(1|sound_file), df) 
summary(mod_arrow)
drop1(mod_arrow, test='Chisq') # L = 2.1, df = 1, p = .15

# So: the type of stimulus (emotionType) or its perceived valence (+/0/-) are not needed in the full model!!! Only the absolute distance from 50 (valence_absolute)
df$nonneg = df$valence_rating >= 50
mod0 = lmer(max_diam~valence_absolute*nonneg+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df)
summary(mod0)
drop1(mod0, test='Chisq')

mod00 = lmer(max_diam~valence_absolute+nonneg+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df)
summary(mod00)
drop1(mod00, test='Chisq')

mod1 = lmer(max_diam~valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df)
summary(mod1)
drop1(mod1, test='Chisq')
plot(mod1)

df$br = df$trial_baseline / df$pre_exp_baseline
mod2 = lmer(max_diam~valence_absolute+confidence_rating*br + trial_baseline+duration+arrow_response_time+(1|pid)+(1|sound_file), df)
summary(mod2)
drop1(mod2, test='Chisq')


## gender, age, condition
mod2 = lmer(max_diam~condition+trial_baseline+(1|pid)+(1|sound_file), df) 
drop1(mod2, test='Chisq')

mod3 = lmer(max_diam~gender+trial_baseline+(1|pid)+(1|sound_file), df) 
drop1(mod3, test='Chisq')

mod4 = lmer(max_diam~age+trial_baseline+(1|pid)+(1|sound_file), df) 
drop1(mod4, test='Chisq')



### Bayesian final model
# mod_brm_max_diam_mm = brm(max_diam~valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df, family='gaussian', chains=4, cores=4, prior=prior(normal(0,5)), iter=5000)
# saveRDS(mod_brm_max_diam_mm, 'mod_brm_max_diam_mm.RDS')
mod_brm_max_diam_mm = readRDS('mod_brm_max_diam_mm.RDS')
# plot(mod_brm_max_diam_mm)
# WAIC(mod_brm_max_diam_mm) 

# for the table: betas times range of each var
s = summary(mod_brm_max_diam_mm)
ss = s$fixed[2:nrow(s$fixed), c(1, 3, 4)] 
for (i in 1:nrow(ss)) {
  ss[i, ] = ss[i, ] * diff(range(df[, rownames(ss)[i]]))
}
soundgen:::reportCI(ss, 2)


delta_int = fitted (mod_brm_max_diam_mm, 
                    newdata=data.frame(valence_absolute = c(0,50),
                                       confidence_rating=median(df$confidence_rating),
                                       duration=median(df$duration),
                                       arrow_response_time=median(df$arrow_response_time),
                                       trial_baseline = median(df$trial_baseline)),
                    re_formula=NA, summary=F)
quantile(delta_int[,2]-delta_int[,1], probs=c(.5, .025, .975)) # pupil response 0.07 mm [0.03, 0.12] greater for sounds with intensity 50 vs. 0

delta_conf = fitted (mod_brm_max_diam_mm, 
                     newdata=data.frame(valence_absolute = median(df$valence_absolute),
                                        confidence_rating=c(0,100),
                                        duration=median(df$duration),
                                        arrow_response_time=median(df$arrow_response_time),
                                        trial_baseline = median(df$trial_baseline)),
                     re_formula=NA, summary=F)
quantile (delta_conf[,2]-delta_conf[,1], probs=c(.5, .025, .975))  # 


newdata = expand.grid (valence_absolute = seq(0,50,length.out=100),
                       confidence_rating=seq(0,100,length.out=100),
                       duration=median(df$duration),
                       arrow_response_time=median(df$arrow_response_time),
                       trial_baseline = median(df$trial_baseline))
f = fitted (mod_brm_max_diam_mm, newdata, re_formula=NA, summary=T)
colnames(f) = c('fit','se','lwr','upr')

df_plot = cbind(newdata, f)
out_pred = as.matrix(dcast(df_plot, valence_absolute~confidence_rating, value.var="fit"))
rownames(out_pred) = out_pred[, 1] # valence_abs in rows
out_pred = out_pred[, -1]

cut2 = function(x, breaks) { # returns the center of bins as labels (breaks based on quantiles)
  s = seq(0, 1, length=2*breaks+1)
  b = quantile(x, probs = s)
  brk = b[0:breaks*2+1]
  mid = b[1:breaks*2]
  brk[1] = brk[1]-0.01
  k = cut(x, breaks=brk, labels=FALSE)
  mid[k]
}

# observed data aggregated in bins
nBins = 8

obs = aggregate(max_diam ~ cut2(valence_absolute, breaks = nBins) + 
                  cut2(confidence_rating, breaks = nBins), 
                df, median)
colnames(obs) = c('valence_absolute', 'confidence_rating', 'max_diam')
obs$duration = median(df$duration)
obs$arrow_response_time = median(df$arrow_response_time)
obs$trial_baseline = median(df$trial_baseline)

# initialize the perspective plot
# jpeg(filename = 'pix/peak~conf_intensity_3D_mm.jpg', width = 15, height = 15, units = 'cm', res = 600, quality = 100)
pp = persp3D(as.numeric(rownames(out_pred)), as.numeric(colnames(out_pred)), out_pred, theta=-45, phi=30, colkey=F, ticktype='detailed', cex.axis=0.75, zlab='Peak pupil dilation, mm', xlab='Valence intensity, %', ylab='Confidence, %', zlim = range(obs$max_diam))

# 3d-2d conversion for adding observed data to the existing perspective plot
elevation_points <- trans3d(obs[,1], obs[,2], obs[,3], pmat=pp)

# calculate line segments connecting observed points with the regression plane
obs$fit = predict(mod_brm_max_diam_mm, newdata=obs, re_formula=NA)[,1]
obs$col = ifelse(obs$fit<obs$max_diam, 'gray30', 'gray70')
points(elevation_points, pch=20, cex=1, col=obs$col)

# add line segments
fit_vals <- trans3d(obs[,1], obs[,2], obs$fit, pmat = pp)
segments(fit_vals$x, fit_vals$y, elevation_points$x, elevation_points$y, lty = 3)
# dev.off()


## Alternatively, show 2D slices, eg the effect of valence in single/coa condition, at low/high confidence (two plots, one for each confidence level), with 95% CI
newdata = expand.grid (valence_absolute = c(0,50),
                       confidence_rating=seq(0,100,length.out=100),
                       duration=median(df$duration),
                       arrow_response_time=median(df$arrow_response_time))
f = fitted (mod_brm_max_diam_mm, newdata, re_formula=NA, summary=T)
colnames(f) = c('fit','se','lwr','upr')
df_plot = cbind(newdata, f)
df_plot$valence_absolute = as.factor(df_plot$valence_absolute)

ggplot(df_plot, aes(x=confidence_rating, y=fit, fill=valence_absolute, ymin=lwr, ymax=upr)) +
  geom_line() +
  geom_ribbon(alpha=.25) +
  ylab('max_diam pupil response, %') +
  theme_bw()
# ggsave ('pix/max_diam~conf_intensity.jpg')

## plotting for duration and arrow_response_time
newdata = expand.grid (valence_absolute = median(df$valence_absolute),
                       confidence_rating=median(df$confidence_rating),
                       duration=c(.5, 5),
                       arrow_response_time=seq(min(df$arrow_response_time), max(df$arrow_response_time), length.out=100))
f = fitted (mod_brm_max_diam_mm, newdata, re_formula=NA, summary=T)
colnames(f) = c('fit','se','lwr','upr')
df_plot = cbind(newdata, f)
df_plot$duration = as.factor(df_plot$duration)

ggplot(df_plot, aes(x=arrow_response_time, y=fit, fill=duration, ymin=lwr, ymax=upr)) +
  geom_line() +
  geom_ribbon(alpha=.25) +
  ylab('max_diam pupil response, %') +
  theme_bw()
# ggsave ('pix/max_diam~dur_arrowRT.jpg')



## Effect of condition
mod_cond = lmer(max_diam~condition+(1|pid)+(1|sound_file), df)
summary (mod_cond)
drop1 (mod_cond, test='Chisq') # L = 1.0, df = 1, p = .30

mod_cond = lmer(time_max_diam~condition+(1|pid)+(1|sound_file), df)
summary (mod_cond)
drop1 (mod_cond, test='Chisq') # L = 2.7, df = 1, p = .10

mod_cond = lmer(auc_toRT~condition+(1|pid)+(1|sound_file), df)
summary (mod_cond)
drop1 (mod_cond, test='Chisq') # L = 1.5, df = 1, p = .22


## Individual variability in the ratings of em. intensity per sound
boxplot(df$valence_rating~df$sound_file)
class(df$sound_file)
names(df)

a = aggregate(valence_rating ~ sound_file, df, mean)
df$valence_aggreg = a$valence_rating[match(df$sound_file, a$sound_file)]

nBins = 15
df$valence_rating_cut = cut(df$valence_rating, nBins)
df$valence_aggreg_cut = cut(df$valence_aggreg, nBins)

a1 = aggregate(max_diam ~ valence_rating_cut, df, mean)
plot(a1)
ggplot(df, aes(x = valence_rating, y = max_diam)) +
  geom_point() +
  geom_smooth()

a2 = aggregate(max_diam ~ valence_aggreg_cut, df, mean)
plot(a2)
ggplot(df, aes(x = valence_aggreg, y = max_diam)) +
  geom_point() +
  geom_smooth()

plot(df$valence_aggreg, df$valence_rating)



##  Did subjects, in practice, wait until sounds had finished playing before responding?
df$resp_minus_offset = df$arrow_response_time - df$duration
hist(df$resp_minus_offset)
mean(df$resp_minus_offset < 0)
mean(df$resp_minus_offset > 1)
1 - mean(df$resp_minus_offset < 0) - mean(df$resp_minus_offset > 1)

plot(df$duration, df$resp_minus_offset)
ggplot(df, aes(x = duration, y = resp_minus_offset)) +
  geom_point() +
  geom_smooth(method = 'lm') +
  geom_hline(yintercept = 0, linetype = 2) +
  xlab('Duration, s') +
  ylab('Response time relative to sound offset, s') +
  scale_x_continuous(breaks = 0:6) +
  annotate('text', x = 3.5, y = -5, label = 'Respond before the sound stops playing') +
  annotate('text', x = 3.5, y = 7.5, label = 'Respond after the sound stops playing') +
  theme_bw()
# ggsave ('pix/RT-minus-offset.jpg', width=15, height=10, units='cm', dpi=300, scale=1.1)



## Does the slope of pupil dilation and return to baseline depend on intensity / confidence / etc?
df1 = df[df$max_diam_time > .25, ]
df1$slope_up = df1$max_diam / df1$max_diam_time
range(df1$max_diam_time)
hist(df1$slope_up)
plot(df1$max_diam, df1$max_diam_time)

mod1 = glmer(slope_up ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df1)
summary(mod1)
drop1(mod1, test = 'Chisq')




