## What predicts the time of peak pupil dilation?

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/Manuel_pupil-response/Analysis')

library (lme4)
library (brms)

df = read.csv('spd_dataset_interp.csv')[,-1]
colnames (df)

# max_diam_time
range(df$max_diam_time)
sum(df$max_diam_time < .1) 

## Final model for max_diam_time
mod_brm_peakTime_base = brm(max_diam_time ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df, family='lognormal', chains=4, cores=4, prior=prior(normal(0,5)), iter=5000)
# saveRDS (mod_brm_peakTime_base, 'mod_brm_peakTime_base') 
# mod_brm_peakTime_base = readRDS ('mod_brm_peakTime_base')
# plot(mod_brm_peakTime_base)
summary(mod_brm_peakTime_base)
# WAIC(mod_brm_peakTime_base) 

# for the table: betas times range of each var
s = summary(mod_brm_peakTime_base)
ss = s$fixed[2:nrow(s$fixed), c(1, 3, 4)] 
for (i in 1:nrow(ss)) {
  ss[i, ] = ss[i, ] * diff(range(df[, rownames(ss)[i]]))
}
reportCI(ss, 2)

# coefs for a table
coda = posterior_samples(mod_brm_peakTime)[,1:5]
a = t (apply (coda, 2, function(x)quantile(x, probs=c(.5,.025,.975))) )
a = round(a, 3)
paste0(a[,1], ' [', a[,2], ', ', a[,3], ']')

exp (a[2,] * 50) # valence_absolute
1 - exp (a[3,] * 100) # confidence
exp (a[4,]) # duration
exp (a[5,]) # arrow response time


# for plotting
newdata = expand.grid (valence_absolute=median(df$valence_absolute),
                       confidence_rating=median(df$confidence_rating),
                       duration=c(.5, 5),
                       arrow_response_time=seq(min(df$arrow_response_time), max(df$arrow_response_time), length.out=100))
f = fitted (mod_brm_peakTime, newdata, re_formula=NA, summary=T)
colnames(f) = c('fit','se','lwr','upr')
f = exp(f)
df_plot = cbind(newdata, f)
df_plot$duration = factor(df_plot$duration, levels=c(5, 0.5))

ggplot(df_plot, aes(x=arrow_response_time, y=fit, linetype=duration, fill=duration, ymin=lwr, ymax=upr)) +
  geom_line() +
  geom_ribbon(alpha=.25) +
  scale_fill_manual (name = 'Sound duration, s', values=c('gray20', 'gray50')) +
  scale_linetype_discrete (name = 'Sound duration, s') +
  xlab('Time to recognition of valence, s') +
  ylab('Time to peak pupil dilation, s') +
  theme_bw() +
  theme (legend.position = c(.2,.8))
# ggsave ('pix/peakTime~dur_arrowRT.jpg', height=10, width=12, units='cm', dpi=300)

# add observed data
cut2 <- function(x, breaks) { # returns the center of bins as labels
  r <- range(x)
  b <- seq(r[1], r[2], length=2*breaks+1)
  brk <- b[0:breaks*2+1]
  mid <- b[1:breaks*2]
  brk[1] <- brk[1]-0.01
  k <- cut(x, breaks=brk, labels=FALSE)
  mid[k]
}

# plot (df$arrow_response_time, df$max_diam_time ) # bin
# df$rt_bin = cut(df$arrow_response_time, breaks=100)
df$rt_bin = cut2(df$arrow_response_time, breaks=100)
ag = aggregate(max_diam_time ~ rt_bin, df, mean)
# plot (ag); range(ag$max_diam_time)

ggplot(df_plot, aes(x=arrow_response_time, y=fit, linetype=duration, fill=duration, ymin=lwr, ymax=upr)) +
  geom_line() +
  geom_ribbon(alpha=.25) +
  scale_fill_manual (name = 'Sound duration, s', values=c('gray20', 'gray50')) +
  scale_linetype_discrete (name = 'Sound duration, s') +
  geom_point (data=ag, aes(x=rt_bin, y=max_diam_time), inherit.aes = F, shape=1) +
  xlab('Time to recognition of valence, s') +
  ylab('Time to peak pupil dilation, s') +
  ylim (range(ag$max_diam_time)) +
  theme_bw() +
  theme (legend.position = c(.2,.8))
# ggsave ('pix/peakTime~dur_arrowRT.jpg', height=10, width=12, units='cm', dpi=300)



# newdata = expand.grid (valence_absolute=c(0,50),
#                        confidence_rating=seq(0,100,by=1),
#                        duration=median(df$duration),
#                        arrow_response_time=median(df$arrow_response_time))
# f = fitted (mod_brm_peakTime, newdata, re_formula=NA, summary=T)
# colnames(f) = c('fit','se','lwr','upr')
# f = exp(f)
# df_plot = cbind(newdata, f)
# df_plot$valence_absolute = factor(df_plot$valence_absolute, levels=c(50,0))
# 
# ggplot(df_plot, aes(x=confidence_rating, y=fit, linetype=valence_absolute, fill=valence_absolute, ymin=lwr, ymax=upr)) +
#   geom_line() +
#   geom_ribbon(alpha=.25) +
#   scale_fill_manual (name = 'Perceived intensity', values=c('gray20', 'gray50')) +
#   scale_linetype_discrete (name = 'Perceived intensity') +
#   xlab('Confidence, %') +
#   ylab('Time to peak pupil dilation, s') +
#   theme_bw() +
#   theme (legend.position = c(.2,.8))






### AUC
cor (df[,c('peak','auc_to10s','auc_toRT','auc_toSoundDur')])

hist (df$auc_to10s)
mod1 = glmer(auc_to10s~valence_absolute+confidence_rating+duration+arrow_response_time+(1|pid)+(1|sound_file)+(1|emotion), df, family='gaussian')
plot(mod1)
summary (mod1)
drop1 (mod1, test='Chisq')
# basically the same as for peak

hist (df$auc_toRT)
mod2 = glmer(auc_toRT~valence_absolute+confidence_rating+duration+arrow_response_time+(1|pid)+(1|sound_file)+(1|emotion), df, family='gaussian')
plot(mod2)
summary (mod2)
drop1 (mod2, test='Chisq') # no effect of confidence, otherwise similar

hist (df$auc_toSoundDur)
mod3 = glmer(auc_toSoundDur~valence_absolute+confidence_rating+duration+arrow_response_time+(1|pid)+(1|sound_file)+(1|emotion), df, family='gaussian')
plot(mod3)
summary (mod3)
drop1 (mod3, test='Chisq') # no effect of confidence, otherwise similar



### Does trial baseline affect the relationship between other predictors and peak pupil response?
mod_base0_time = lmer(max_diam_time_log ~ valence_absolute+confidence_rating+duration+arrow_response_time+(1|pid)+(1|sound_file), data = df)
summary(mod_base0_time)
drop1(mod_base0_time, test = "Chisq")

mod_base1_time = lmer(max_diam_time_log ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), data = df)
summary(mod_base1_time)
drop1(mod_base1_time, test = "Chisq")
# thus: baseline is a highly significant (negative) predictor of the time of peak pupil response (~20% slower for each mm change in baseline pupil diameter), but the other predictors hold even after controlling for baseline (interaction not so interesting)

plot(df$trial_baseline, df$max_diam_time)
