## Pupil response curves as time series

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/006_with-Manuel/analysis')

library (zoo)
library (brms)
library (mgcv)
library (plot3D)
library (reshape2)
library (data.table)
library (lme4)

df = read.csv('spd_dataset_interp.csv')[,-1]
colnames(df)
df$trial_samples = as.character(df$trial_samples)

df$soundOffDif = df$max_diam_time - df$duration
df$arrowTimeDif = df$max_diam_time - df$arrow_response_time
df$soundOnDif = df$max_diam_time

## remove neutral sounds:
# df = df[df$emotion != 'neutral', ]
## remove sexual sounds:
# df = df[!df$erotic, ]
## preserve only confidence > 80%
# df = df[df$confidence_rating > 80, ]

## set pars for extraction of pupil curves
samplingRate_new = 30
dur_limits = c(0, Inf) # dur_limits = c(1.5, 2.5)
my_outcome = c('soundOffDif', 'arrowTimeDif', 'soundOnDif')[3] # time-lock to which event?
time_post_peak = 2 # record value at ... s after the peak

##


downsample = function(s, sr=10, sr_original=120){
  if (!sr < sr_original){
    return (s)
  }
  l = length(s)
  dur = l / sr_original
  idx = seq (1, l, length.out=round(dur*sr))
  return (s[idx])
}
# downsample (s=1:1000, sr=5, sr_original=100)

## preserve individual pupil response curves
idx = which(df$duration>dur_limits[1] & df$duration<dur_limits[2])
out_list = list()
for (i in idx){
  a = noquote(df$trial_samples[i])
  a = substr(a, 2, nchar(a)-1)
  a = as.numeric(unlist(strsplit(a, ', ')))
  # a = (a - df$trial_baseline[i]) / df$pre_exp_baseline [i] * 100
  
  if (my_outcome=='soundOffDif'){
    myProportion = (df$duration[i]*30 + 39) / length(a)            # location of the end of sound
  } else if (my_outcome=='arrowTimeDif'){
    myProportion = (df$arrow_response_time[i]*30 + 39) / length(a) # location of arrow press
  } else if (my_outcome=='soundOnDif'){
    myProportion = 39 / length(a)                                  # location of the beginning of sound 
  } 
  
  if (samplingRate_new < 30){
    a = rollmean(a, k=11)
    a = downsample(a, sr_original=30, sr=samplingRate_new)
  }
  # plot (a, type='l')
  
  myIndex = round(myProportion * length(a)) # time-locking point
  temp = data.frame(sound=rep(df$sound_file_name[i],length(a)),
                    participant = df$pid[i],
                    trialID = paste0('trial_',i),
                    valence = df$sound_valence[i],
                    valence_absolute = df$valence_absolute[i],
                    valence_rating = df$valence_rating[i],
                    subjective_absolute = df$subjective_absolute[i],
                    subjective_rating = df$subjective_rating[i],
                    confidence_rating = df$confidence_rating[i], # df$confidence_rating_logit[i],
                    trial_baseline = df$trial_baseline[i],
                    pre_exp_baseline = df$pre_exp_baseline[i],
                    pupil_response = df$pupil_response[i],
                    emotion = df$emotion[i],
                    duration = df$duration[i],
                    time = (1 : length(a)) - myIndex, # time-locked to the measure of interest
                    pupil = a) # plot (temp$time, temp$pupil, type='l')
  df$length_post_peak[i] = length(a)/30 - df$max_diam_time[i]
  df$peakTime[i] = temp$time[which.max(a[40:length(a)]) + 39] / 30 # check with Manuel: why does this differ from df$max_diam_dime ???
  df$value_post_peak[i] = a[which.max(a) + time_post_peak * 30]  
  df$value_at_arrow[i] = a[round(df$arrow_response_time[i] * 1000 / 30) + 39]
  df$value_post_arrow[i] = a[round(df$arrow_response_time[i] * 1000 / 30) + 39 + time_post_peak * 30]
  df$time_post_arrow[i] = (length(a) - round(df$arrow_response_time[i] * 1000 / 30)) * 30 / 1000
  # df$value_post_peak[i] = a[round((df$max_diam_time[i] + time_post_peak) * 30)]
  df$value_at_peak[i] = max(a)
  out_list[[i]] = temp
  if (i %% 100 == 0) print (i)
}
out = data.table::rbindlist(out_list)
out$time = out$time/samplingRate_new # convert time stamps from points to seconds

# plot(df$max_diam, df$value_at_peak)  # minor differences, b/c Manuel extracted max_diam before smoothing (presumably)

# # plot everything
# a = aggregate (pupil~time, out, mean)
# plot (a$time, a$pupil, type='l')

ylim = c(-3.5, 15)
breaks = seq(-4, 15, 1)
### Plot of pupil response (mm) - baseline as a function of (1) valence intensity (low / high) and (2) trial baseline divided by pre-experiment baseline (low / medium / high):
legend_labels_base = c('Low (<94%)', 'Medium', 'High (>105%)')
legend_labels_vr = c('Low (0...25)', 'High (25...50)')

out = out[out$time > -5 & out$time < 8,] # discard values very far from the region of interest
out$base_relative = out$trial_baseline / out$pre_exp_baseline * 100
q = quantile(out$base_relative, probs = c(.25, .75))
# hist(out$base_relative)

out$base_bin = cut(out$base_relative, breaks=c(0, 94, 105, +Inf)) # table(out$base_bin)
out$vr_bin = cut(out$valence_absolute, breaks=2) # table(out$vr_bin)
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
a = aggregate(peak_base_div_base ~ time+base_bin+vr_bin, out, mean)
a$se = aggregate(peak_base_div_base ~ time+base_bin+vr_bin, out, function(x)sd(x) / sqrt(length(x)))[,4]

ggplot(a, aes(x=time, y=peak_base_div_base, col=base_bin, linetype=vr_bin, ymin=peak_base_div_base-se, ymax=peak_base_div_base+se)) +
  geom_line() +  # 
  geom_ribbon(aes(fill=base_bin), alpha=.2, col='white', show.legend = F) +
  geom_vline(xintercept = 0, linetype=2) +
  scale_color_discrete(name='Relative baseline', labels=legend_labels_base) +
  scale_linetype_discrete(name='Valence intensity', labels=legend_labels_vr) +
  scale_x_continuous(breaks=seq(floor(min(a$time)), ceiling(max(a$time)), by=1)) +
  scale_y_continuous(limits=ylim, breaks = breaks) +
  xlab('Time from audio onset, s') + 
  ylab('Pupil response, %') +
  theme_bw() + 
  theme(panel.grid = element_blank())

# ggsave('pix/fig2C.jpg', width=18, height=10, units='cm', dpi=600, scale=0.8)



### same but time-locked to arrow
legend_labels_base = c('Low (<94%)', 'Medium', 'High (>105%)')
legend_labels_vr = c('Low (0...25)', 'High (25...50)')

out = out[out$time > -5 & out$time < 4.2,] # discard values very far from the region of interest
out$base_relative = out$trial_baseline / out$pre_exp_baseline * 100
q = quantile(out$base_relative, probs = c(.25, .75))
# hist(out$base_relative)

out$base_bin = cut(out$base_relative, breaks=c(0, 94, 105, +Inf)) # table(out$base_bin)
out$vr_bin = cut(out$valence_absolute, breaks=2) # table(out$vr_bin)
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
a = aggregate(peak_base_div_base ~ time+base_bin+vr_bin, out, mean)
a$se = aggregate(peak_base_div_base ~ time+base_bin+vr_bin, out, function(x)sd(x) / sqrt(length(x)))[,4]

ggplot(a, aes(x=time, y=peak_base_div_base, col=base_bin, linetype=vr_bin, ymin=peak_base_div_base-se, ymax=peak_base_div_base+se)) +
  geom_line() +  # 
  geom_ribbon(aes(fill=base_bin), alpha=.2, col='white', show.legend = F) +
  geom_vline(xintercept = 0, linetype=2) +
  scale_color_discrete(name='Relative baseline', labels=legend_labels_base) +
  scale_linetype_discrete(name='Valence intensity', labels=legend_labels_vr) +
  scale_x_continuous(breaks=seq(floor(min(a$time)), ceiling(max(a$time)), by=1)) +
  scale_y_continuous(position = "right", limits = ylim, breaks = breaks) +
  xlab('Time from pressing response key, s') + 
  ylab('Pupil response, %') +
  theme_bw() + 
  theme(panel.grid = element_blank())

# ggsave('pix/fig2D.jpg', width=18, height=10, units='cm', dpi=600, scale=0.8)


## time-locked to audio onset, by valence (like the old plot)
ylim = c(-1.5, 11.5)
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
legend_labels = c('Very negative\n(-50...-30)', 'Negative\n(-30...-10)', 'Neutral\n(-10...+10)', 'Positive\n(+10...+30)', 'Very positive\n(+30...+50)')

out$vr_bin = cut(out$valence_rating, breaks=5) # table(out$vr_bin)
a = aggregate(peak_base_div_base~time+vr_bin, out, mean)
a$se = aggregate(peak_base_div_base~time+vr_bin, out, function(x)sd(x) / sqrt(length(x)))[,3]
a = a[a$time > -5 & a$time < 8, ]

ggplot (a, aes(x=time, y=peak_base_div_base, ymin=peak_base_div_base-se, ymax=peak_base_div_base+se, col=vr_bin, linetype=vr_bin)) +
  geom_line() +
  geom_ribbon(aes(fill=vr_bin), alpha=.2, col='white', show.legend = F) +
  geom_vline(xintercept = 0, linetype=2) +
  scale_color_discrete(name='Valence', labels=legend_labels) +
  scale_linetype_discrete(name='Valence', labels=legend_labels) +
  scale_x_continuous(breaks=seq(floor(min(a$time)), ceiling(max(a$time)), by=1)) +
  scale_y_continuous(breaks = breaks, limits = ylim) +
  xlab('Time from audio onset, s') + 
  ylab('Pupil response, %') +
  theme_bw() + 
  theme(legend.position = 'right',
        legend.key.size = unit(2, 'lines'),
        legend.text.align = 0,
        legend.title.align = 0.5,
        panel.grid = element_blank())

# ggsave('pix/fig2A.jpg', width=18, height=10, units='cm', dpi=600, scale=0.8)


## time-locked to arrow, by valence (like the old plot)
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
legend_labels = c('Very negative\n(-50...-30)', 'Negative\n(-30...-10)', 'Neutral\n(-10...+10)', 'Positive\n(+10...+30)', 'Very positive\n(+30...+50)')

out$vr_bin = cut(out$valence_rating, breaks=5) # table(out$vr_bin)
a = aggregate(peak_base_div_base~time+vr_bin, out, mean)
a$se = aggregate(peak_base_div_base~time+vr_bin, out, function(x)sd(x) / sqrt(length(x)))[,3]
a = a[a$time > -5 & a$time < 4.2, ]

ggplot (a, aes(x=time, y=peak_base_div_base, ymin=peak_base_div_base-se, ymax=peak_base_div_base+se, col=vr_bin, linetype=vr_bin)) +
  geom_line() +
  geom_ribbon(aes(fill=vr_bin), alpha=.2, col='white', show.legend = F) +
  geom_vline(xintercept = 0, linetype=2) +
  scale_color_discrete(name='Valence', labels=legend_labels) +
  scale_linetype_discrete(name='Valence', labels=legend_labels) +
  scale_x_continuous(breaks=seq(floor(min(a$time)), ceiling(max(a$time)), by=1)) +
  scale_y_continuous(position = "right", breaks = breaks, limits = ylim) +
  xlab('Time from pressing response key, s') + 
  ylab('Pupil response, %') +
  theme_bw() + 
  theme(legend.position = 'right',
        legend.key.size = unit(2, 'lines'),
        legend.text.align = 0,
        legend.title.align = 0.5,
        panel.grid = element_blank())

# ggsave('pix/fig2B.jpg', width=18, height=10, units='cm', dpi=600, scale=0.8)


### plot by confidence
filename = paste0('pix/peak-base-div-base_conf.jpg')
out$base_relative = out$trial_baseline / out$pre_exp_baseline * 100
q = quantile(out$base_relative, probs = c(.25, .75))
hist(out$base_relative)
quantile(out$confidence_rating, probs = c(.25, .5, .75))

out = out[out$time > -5 & out$time < 8,] # discard values very far from the region of interest
out$base_bin = cut(out$base_relative, breaks=c(0, 94, 105, +Inf)) # table(out$base_bin)
out$cf_bin = cut(out$confidence_rating, breaks=c(0, 73, 100)) # table(out$cf_bin)
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
a = aggregate(peak_base_div_base ~ time+base_bin+cf_bin, out, mean)
a$se = aggregate(peak_base_div_base ~ time+base_bin+cf_bin, out, function(x)sd(x) / sqrt(length(x)))[,4]

legend_labels_base = c('Low (<94%)', 'Medium', 'High (>105%)')
legend_labels_vr = c('Low (0...73)', 'High (73...100)')

ggplot(a, aes(x=time, y=peak_base_div_base, col=base_bin, linetype=cf_bin, ymin=peak_base_div_base-se, ymax=peak_base_div_base+se)) +
  geom_line() +  # 
  geom_ribbon(aes(fill=base_bin), alpha=.2, col='white', show.legend = F) +
  geom_vline(xintercept = 0, linetype=2) +
  scale_color_discrete(name='Relative baseline', labels=legend_labels_base) +
  scale_linetype_discrete(name='Confidence', labels=legend_labels_vr) +
  scale_x_continuous(breaks=seq(floor(min(a$time)), ceiling(max(a$time)), by=1)) +
  # scale_y_continuous(breaks=-5:12, limits=ylim) +
  xlab('Time from audio onset, s') + ylab('Pupil response, %') +
  theme_bw() + 
  theme(panel.grid = element_blank())

# ggsave(pix/peak-base-div-base.jpg, width=15, height=10, units='cm', dpi=300, scale=1.1)



### plot for time-locking to arrow press
legend_labels_base = c('Low (<94%)', 'Medium', 'High (>105%)')
legend_labels_vr = c('Low (0...25)', 'High (25...50)')

out = out[out$time > -5 & out$time < 8,] # discard values very far from the region of interest
xlab = 'Time from pressing response key, s'

out$base_relative = out$trial_baseline / out$pre_exp_baseline * 100
q = quantile(out$base_relative, probs = c(.25, .75))
hist(out$base_relative)

out$base_bin = cut(out$base_relative, breaks=c(0, 94, 105, +Inf)) # table(out$base_bin)
out$vr_bin = cut(out$valence_absolute, breaks=2) # table(out$vr_bin)
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
a = aggregate(peak_base_div_base ~ time+base_bin+vr_bin, out, mean)
a$se = aggregate(peak_base_div_base ~ time+base_bin+vr_bin, out, function(x)sd(x) / sqrt(length(x)))[,4]

ggplot(a, aes(x=time, y=peak_base_div_base, col=base_bin, linetype=vr_bin, ymin=peak_base_div_base-se, ymax=peak_base_div_base+se)) +
  geom_line() +  # 
  geom_ribbon(aes(fill=base_bin), alpha=.2, col='white', show.legend = F) +
  geom_vline(xintercept = 0, linetype=2) +
  scale_color_discrete(name='Relative baseline', labels=legend_labels_base) +
  scale_linetype_discrete(name='Valence intensity', labels=legend_labels_vr) +
  scale_x_continuous(breaks=seq(floor(min(a$time)), ceiling(max(a$time)), by=1)) +
  # scale_y_continuous(breaks=-5:12, limits=ylim) +
  xlab(xlab) + ylab('Pupil response, %') +
  theme_bw() + 
  theme(panel.grid = element_blank())

# ggsave('pix/peak-base-div-base_lockToArrow.jpg', width=15, height=10, units='cm', dpi=300, scale=1.1)



### plot by confidence_rating
# legend_labels = c('Low (0...50)', 'High (50...100)') # legend_labels = 1:5
legend_labels = c('0...20', '20...40', '40...60', '60...80', '80...100')
# legend_labels = c('0...6', '6...28', '28...71', '71...94', '94...100')

out$vr_bin = cut(out$confidence_rating, breaks=5) # table(out$vr_bin)
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
a = aggregate(peak_base_div_base~time+vr_bin, out, mean)
a$se = aggregate(peak_base_div_base~time+vr_bin, out, function(x)sd(x) / sqrt(length(x)))[,3]
a = a[a$time < 8, ]

ggplot (a, aes(x=time, y=peak_base_div_base, ymin=peak_base_div_base-se, ymax=peak_base_div_base+se, col=vr_bin, linetype=vr_bin)) +
  geom_line() +
  geom_ribbon(aes(fill=vr_bin), alpha=.2, col='white', show.legend = F) +
  geom_vline(xintercept = 0, linetype=2) +
  scale_color_discrete(name='Confidence', labels=legend_labels) +
  scale_linetype_discrete(name='Confidence', labels=legend_labels) +
  scale_x_continuous(breaks=seq(floor(min(a$time)), ceiling(max(a$time)), by=1)) +
  # scale_y_continuous(breaks=-5:12, limits=ylim) +
  xlab(xlab) + ylab('Pupil response, %') +
  theme_bw() + 
  theme(legend.position = c(.42, .21),
        panel.grid = element_blank())

# ggsave('pix/confidence.jpg', width=15, height=10, units='cm', dpi=300, scale=1.1)


## 3D plots
out$peak_base_div_base = (out$pupil - out$trial_baseline) / out$trial_baseline * 100
norm_coef = diff(range(out$time))/100 # diff(range(out$valence_rating))
out$valence_rating = out$valence_rating * norm_coef # to have time and valence_rating on the same scale and use s() instead of te()
mod_gam = gam(peak_base_div_base~s(time, valence_rating, k=50) + trial_baseline, data=out) # ignores autocorrelation, so not to be used for inference, just for plotting the trend
# mod_gam = gam(pupil~s(time, valence_rating, k=50), data=out) # ignores autocorrelation, so not to be used for inference, just for plotting the trend
plot(mod_gam)

newdata = expand.grid(
  time = seq(-max(out$time), max(out$time), length.out = 500), # seq(min(out$time), max(out$time), length.out=100),
  valence_rating = seq(0, 100, length.out=500)*norm_coef,
  trial_baseline = median(df$trial_baseline))
df_plot = cbind(newdata, predict(mod_gam, newdata))
colnames(df_plot) = c('time', 'valence_rating', 'trial_baseline', 'fit')
df_plot$valence_rating = df_plot$valence_rating/norm_coef - 50 # convert to -50...+50
d = as.matrix(dcast(df_plot, time ~ valence_rating, value.var="fit"))
rownames(d) = d[,1] # time in rows
d = d[,-1]

filename = 'pix/3D.jpg' # filename = 'pix_pupil_ts/persp3D_lockedToArrowResponse_noNeutral.jpg'
valNeutral_idx = which.min(abs(as.numeric(colnames(d))))
time_zero_idx = which.min(abs(as.numeric(rownames(d))))
timeRange = range(as.numeric(rownames(d)))

# jpeg(filename, width=2000, height=2000, res=300)
par(mar = c(1, 0, 0, 0))
plot3D::persp3D(as.numeric(rownames(d)), as.numeric(colnames(d)), d, theta=-20, phi=40, zlab='Pupil response, %', xlab='Time, s', ylab='Valence, %', colkey=F, ticktype='detailed', cex.axis=0.75, mar=c(5,0,0,0))
plot3D::lines3D(x=seq(timeRange[1],timeRange[2],length.out=nrow(d)), y=rep(0,nrow(d)), z=d[,valNeutral_idx], col='black', lwd=4, lty=1, add=T)
plot3D::lines3D(x=rep(0,nrow(d)), y=seq(-50,50,length.out=nrow(d)), z=d[time_zero_idx,], col='black', lwd=4, lty=1, add=T) # show the moment of response
plot3D::segments3D(x0=rep(0,40), x1=rep(0,40), y0=seq(-50,50,length.out=40), y1=seq(-50,50,length.out=40), z0=rep(0,40), z1=d[time_zero_idx, seq(1, ncol(d), length.out=40)], col='gray30', lwd=1, lty=3, add=T)
par(mar = c(5.1, 4.1, 4.1, 2.1)) # back to defaults
# dev.off()



### Does the slope of pupil dilation and return to baseline depend on intensity / confidence / etc?
## SLOPE UP
df1 = df[df$max_diam_time > .25, ]
df1$slope_up = (df1$max_diam - df1$trial_baseline) / df1$max_diam_time
range(df1$max_diam_time)
hist(df1$slope_up)

mod1_2 = glmer(slope_up ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df1)
summary(mod1_2)
# plot(mod1_2)
drop1(mod1_2, test = 'Chisq')

plot(df1$trial_baseline, df1$slope_up)

mod_brm_slope_up = brm(slope_up~valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df1, family='gaussian', chains=4, cores=4, prior=prior(normal(0,5)), iter=5000)
# saveRDS(mod_brm_slope_up, 'mod_brm_slope_up')
# mod_brm_slope_up = readRDS('mod_brm_slope_up')
summary(mod_brm_slope_up)
# plot(mod_brm_slope_up)
# WAIC(mod_brm_slope_up) 

# for the table: betas times range of each var
s = summary(mod_brm_slope_up)
ss = s$fixed[2:nrow(s$fixed), c(1, 3, 4)] 
for (i in 1:nrow(ss)) {
  ss[i, ] = ss[i, ] * diff(range(df[, rownames(ss)[i]]))
}
reportCI(ss, 2)


## SLOPE DOWN
df$slope_down = (df$value_post_peak - df$value_at_peak) / time_post_peak
hist(df$slope_down)
summary(df$slope_down)

mod3 = glmer(slope_down ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+(1|pid)+(1|sound_file), df)
summary(mod3)
plot(mod3)
drop1(mod3, test = 'Chisq')

mod4 = glmer(slope_down ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+value_at_peak+(1|pid)+(1|sound_file), df)
summary(mod4)
drop1(mod4, test = 'Chisq')

df$slope_down_log = -log(-df$slope_down)
mod5 = glmer(slope_down_log ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+value_at_peak+(1|pid)+(1|sound_file), df)
summary(mod5)
plot(mod5)
drop1(mod4, test = 'Chisq')


mod_brm_slope_down = brm(slope_down_log ~ valence_absolute+confidence_rating+duration+arrow_response_time+trial_baseline+value_at_peak+(1|pid)+(1|sound_file), df, family='gaussian', chains=4, cores=4, prior=prior(normal(0,5)), iter=5000)
# saveRDS(mod_brm_slope_down, 'mod_brm_slope_down')
# mod_brm_slope_down = readRDS('mod_brm_slope_down')
summary(mod_brm_slope_down)
# plot(mod_brm_slope_down)
# WAIC(mod_brm_slope_down) 

# for the table: betas times range of each var
f = fitted(mod_brm_slope_down, 
           newdata = data.frame(valence_absolute = range(df$valence_absolute),
                                confidence_rating = median(df$confidence_rating),
                                duration = median(df$duration),
                                arrow_response_time = median(df$arrow_response_time),
                                trial_baseline = median(df$trial_baseline),
                                value_at_peak = median(df$value_at_peak)),
           summary = FALSE, re_formula = NA)
f = exp(-f) # back to -mm/s
reportCI(quantile(f[, 2] - f[, 1], probs = c(.5, .025, .975)), 2) 

f = fitted(mod_brm_slope_down, 
           newdata = data.frame(valence_absolute = median(df$valence_absolute),
                                confidence_rating = range(df$confidence_rating),
                                duration = median(df$duration),
                                arrow_response_time = median(df$arrow_response_time),
                                trial_baseline = median(df$trial_baseline),
                                value_at_peak = median(df$value_at_peak)),
           summary = FALSE, re_formula = NA)
f = exp(-f) # back to -mm/s
reportCI(quantile(f[, 2] - f[, 1], probs = c(.5, .025, .975)), 2) 

f = fitted(mod_brm_slope_down, 
           newdata = data.frame(valence_absolute = median(df$valence_absolute),
                                confidence_rating = median(df$confidence_rating),
                                duration = range(df$duration),
                                arrow_response_time = median(df$arrow_response_time),
                                trial_baseline = median(df$trial_baseline),
                                value_at_peak = median(df$value_at_peak)),
           summary = FALSE, re_formula = NA)
f = exp(-f) # back to -mm/s
reportCI(quantile(f[, 2] - f[, 1], probs = c(.5, .025, .975)), 2) 

f = fitted(mod_brm_slope_down, 
           newdata = data.frame(valence_absolute = median(df$valence_absolute),
                                confidence_rating = median(df$confidence_rating),
                                duration = median(df$duration),
                                arrow_response_time = range(df$arrow_response_time),
                                trial_baseline = median(df$trial_baseline),
                                value_at_peak = median(df$value_at_peak)),
           summary = FALSE, re_formula = NA)
f = exp(-f) # back to -mm/s
reportCI(quantile(f[, 2] - f[, 1], probs = c(.5, .025, .975)), 2) 

f = fitted(mod_brm_slope_down, 
           newdata = data.frame(valence_absolute = median(df$valence_absolute),
                                confidence_rating = median(df$confidence_rating),
                                duration = median(df$duration),
                                arrow_response_time = median(df$arrow_response_time),
                                trial_baseline = range(df$trial_baseline),
                                value_at_peak = median(df$value_at_peak)),
           summary = FALSE, re_formula = NA)
f = exp(-f) # back to -mm/s
reportCI(quantile(f[, 2] - f[, 1], probs = c(.5, .025, .975)), 2) 

f = fitted(mod_brm_slope_down, 
           newdata = data.frame(valence_absolute = range(df$valence_absolute),
                                confidence_rating = median(df$confidence_rating),
                                duration = median(df$duration),
                                arrow_response_time = median(df$arrow_response_time),
                                trial_baseline = median(df$trial_baseline),
                                value_at_peak = median(df$value_at_peak)),
           summary = FALSE, re_formula = NA)
f = exp(-f) # back to -mm/s
reportCI(quantile(f[, 2] - f[, 1], probs = c(.5, .025, .975)), 2) 





