## What predicts the lag between peak pupil response and pressing the response key?

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/006_with-Manuel/analysis')

library(lme4)
library(ggplot2)
library(brms)

df = read.csv('spd_dataset_interp.csv')[,-1]
df$arrowTimeDif = df$max_diam_time - df$arrow_response_time

hist(df$arrowTimeDif)
a = aggregate(arrowTimeDif ~ cut(valence_rating,breaks=20), df, mean)
plot(a)

b = aggregate(arrowTimeDif ~ cut(valence_absolute,breaks=20), df, mean)
plot(b)

c = aggregate(arrowTimeDif ~ cut(confidence_rating,breaks=20), df, mean)
plot(c)

mod_gam = gam(arrowTimeDif ~ s(valence_rating), data=df)
plot(mod_gam)
newdata = data.frame(valence_rating = seq(0, 100, by=1))
df_plot = cbind(newdata,
                predict(mod_gam, newdata=newdata, se.fit = TRUE))
colnames(df_plot) = c('valence_rating', 'fit', 'se')
ggplot(df_plot, aes(x = valence_rating, y = fit, ymin = fit-se, ymax = fit+se)) +
  geom_line() +
  geom_ribbon(alpha = .25)

mod_brm_gam = brm(arrowTimeDif ~ s(valence_rating)+(1|pid)+(1|sound_file), data=df, family='gaussian', chains=4, cores=4, prior=prior(normal(0,5)), iter=5000)
summary(mod_brm_gam)
df_plot = cbind(newdata,
                predict(mod_brm_gam, newdata=newdata, re_formula=NA))

colnames(df_plot) = c('valence_rating', 'fit', 'se', 'low', 'high')
ggplot(df_plot, aes(x = valence_rating, y = fit, ymin = low, ymax = high)) +
  geom_line() +
  geom_ribbon(alpha = .25)

mod_dur = lmer(arrowTimeDif ~ duration + (1|pid) + (1|sound_file), data=df)
summary(mod_dur)
drop1(mod_dur, test = 'Chisq')

mod1 = lmer(arrowTimeDif ~ valence_absolute + confidence_rating + duration + trial_baseline + (1|pid) + (1|sound_file), data=df)
summary(mod1)
drop1(mod1, test = 'Chisq')

mod2 = lmer(arrowTimeDif ~ valence_absolute + confidence_rating + (1|pid) + (1|sound_file), data=df)
summary(mod2)
drop1(mod2, test = 'Chisq')

mod3 = lmer(arrowTimeDif ~ valence_absolute * confidence_rating + (1|pid) + (1|sound_file), data=df)
summary(mod3)
drop1(mod3, test = 'Chisq')  # no interaction

mod_brm = brm(arrowTimeDif ~ valence_absolute + confidence_rating + trial_baseline + (1|pid) + (1|sound_file), data=df, chains=4, cores=4, prior=prior(normal(0,5)), iter=500)  # only takes a couple of min
# saveRDS(mod_brm, 'mod_brm_lag_per_val_conf.RDS')
# mod_brm = readRDS('mod_brm_lag_per_val_conf.RDS')
s = summary(mod_brm)
s$fixed[2, c(1, 3, 4)] * 50  # effect of valence_absolute over its entire range
s$fixed[3, c(1, 3, 4)] * 100  # effect of confidence_rating over its entire range

brms::marginal_effects(mod_brm)


## cor between peak dilation time and arrow press time
range(df$max_diam_time)
hist(df$max_diam_time)
hist(log(df$max_diam_time))
plot(density(df$max_diam_time))
nrow(df[df$max_diam_time <= .5, ]) / nrow(df) * 100
df = df[df$max_diam_time > .5, ]  # min 500 ms

range(df$arrow_response_time)
hist(df$arrow_response_time)
hist(log(df$arrow_response_time))

cor(df$arrow_response_time, df$max_diam_time)
plot(df$arrow_response_time, df$max_diam_time)
cor(log(df$arrow_response_time), log(df$max_diam_time))
plot(log(df$arrow_response_time), log(df$max_diam_time))

plot(df$arrow_response_time, df$max_diam_time, col=cut(df$valence_absolute, breaks = 4), pch = 16)
plot(df$arrow_response_time, df$max_diam_time, col=cut(df$duration, breaks = 4), pch = 16)

ggplot(df, aes(x = arrow_response_time, max_diam_time, col = duration)) +
  geom_point(alpha = .5) +
  scale_color_continuous(low = 'blue', high = 'red')

ggplot(df, aes(x = arrow_response_time, max_diam_time, col = valence_absolute)) +
  geom_point(alpha = .5) +
  scale_color_continuous(low = 'blue', high = 'red')

mod = lm(max_diam_time ~ arrow_response_time, df)
plot(mod)

mod = lm(log(max_diam_time) ~ log(arrow_response_time), df)
plot(mod)
summary(mod)

mod = lmer(log(max_diam_time) ~ log(arrow_response_time) + (1|pid) + (1|sound_file), df)
summary(mod)
drop1(mod, test = 'Chisq')

mod_brm = brm(log(max_diam_time) ~ log(arrow_response_time) + (1|pid) + (1|sound_file), data=df, chains=4, cores=4, prior=prior(normal(0,5)), iter=5000)  # only takes a couple of min
# saveRDS(mod_brm, 'mod_brm_cor-peak-arrow.RDS')
# mod_brm = readRDS('mod_brm_cor-peak-arrow.RDS')
summary(mod_brm)
plot(mod_brm)
# possible graph: scatterplot + regression curve with 95% CI

